<?php
namespace Service;

use Util\Utilities;
use Util\Instance\RepositoryInstance;
use Util\Instance\GeneralProvider;
use Util\Instance\FactoryInstance;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment as TeknisaEnvironment;
use Teknisa\Libs\Util\Utilities as TeknisaUtilities;
use Teknisa\Model\Teknisa\Entities;
use Zeedhi\Framework\Remote\FileUploader\Server;



class Structure {
    const DOUBLE_BREAK_LINE = '

';
    protected $entityManager;
    private $zhFileUploader;

    public function __construct(
        EntityManager $entityManager, TeknisaEnvironment $environment, Server $zhFileUploader
    ){
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->zhFileUploader = $zhFileUploader;
    }
    public function findStructureWithoutFilter($filters, $page, $pageSize, $orderBy){
        $conditions = Utilities::createQueryFiltered($filters);
        $limit      = $page * $pageSize;
        $offset     = ($page - 1) * $pageSize;
        $order      = '';
        if(empty($orderBy)) {
            $order = "ORDER BY NRSCRIPT DESC NULLS FIRST, ID";
        } else {
            foreach($orderBy as $idx => $order) {
                $order = ' ORDER BY ' . $idx . ' ' . $order;
            }
        }
        
    }
    public function findMtDataBase($filters, $page, $pageSize, $orderBy)
    {
        $conditions = Utilities::createQueryFiltered($filters);
        $limit      = $page * $pageSize;
        $offset     = ($page - 1) * $pageSize;
        return RepositoryInstance::getStructure()->findMtDataBases($conditions, $limit, $offset);
    }

    public function findStructure($filters, $page, $pageSize, $orderBy)
    {
        foreach($filters as &$filter){
            if($filter['columnName'] == 'ISSUES'){
                $filter['columnName'] = 'ISSUE_FILTER';
            }
            if($filter['columnName'] == 'SCRIPT_FILTER'){
                $filter['columnName'] = 'SCRIPT';
            }
            if($filter['columnName'] == 'DT_FILTER'){
                $filter['columnName'] = 'DTULTATU';
            }
        }
        $issueFilterComplement = '';
        $issueFilterConditions = '';
        $issueFilter = Utilities::getValueInFilterDataSourceByName($filters,'ISSUE_FILTER',true);
        if($issueFilter){
            $issueFilter = implode(",", $issueFilter);
            $issueFilter = 'AND TAS.TASK IN (' . $issueFilter . ')';
            $issueFilterConditions = 'AND ISSUE_FILTER IS NOT NULL';
        }
        
        
        $conditions = Utilities::createQueryFiltered($filters);
        $limit      = $page * $pageSize;
        $offset     = ($page - 1) * $pageSize;
        $order      = '';
        if(empty($orderBy)) {
            $order = "ORDER BY NRSCRIPT DESC NULLS FIRST, ID";
        } else {
            foreach($orderBy as $idx => $order) {
                $order = ' ORDER BY ' . $idx . ' ' . $order;
            }
        }
        return RepositoryInstance::getStructure()->getStructure($conditions,$issueFilter,$issueFilterConditions);
    }

    public function findRepositorioById($filters, $page, $pageSize)
    {
        $conditions = Utilities::createQueryFiltered($filters);
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return RepositoryInstance::getStructure()->findRepositorioById($conditions, $limit, $offset);
    }

    public function findIssuesRelatedToTask($filters, $page, $pageSize)
    {
        $currentUser = $this->environment->getCurrentUserId();
        $conditions = Utilities::createQueryFiltered($filters);
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return RepositoryInstance::getStructure()->findIssuesRelatedToTask($conditions, $limit, $offset, $currentUser);
    }

    public function findRelatedScript($nrEstrut)
    {
        $estruturas = RepositoryInstance::getStructure()->findRelatedScript($nrEstrut);
        $row = [];
        $row['DB_ID'] = [];
        $row['DB_NAMES'] = [];
        foreach($estruturas as $estrutura){
            $row['DB_ID_' . $estrutura['DB_ID']] = $estrutura['DB_ID'];
            $row['ID_' . $estrutura['DB_ID']] = $estrutura['ID'];
            $row['SCRIPT_' . $estrutura['DB_ID']] = $estrutura['SCRIPT'];
            $row['IDRESULTSCRIPT_' . $estrutura['DB_ID']] = $estrutura['IDRESULTSCRIPT'];
            $row['NRSCRIPT_' . $estrutura['DB_ID']] = $estrutura['NRSCRIPT'];
            $row['DSDIRLOG_' . $estrutura['DB_ID']] = $estrutura['DSDIRLOG'];
            array_push($row['DB_ID'], $estrutura['DB_ID']);
            array_push($row['DB_NAMES'], $estrutura['NAME']);
            
        }
        return $row;
    }

    public function findIssues($filters, $page, $pageSize)
    {
        $conditions = Utilities::createQueryFiltered($filters);
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return RepositoryInstance::getStructure()->findIssues($conditions, $limit, $offset);
    }


    public function findOperadorById($filters, $page, $pageSize)
    {
        $conditions = Utilities::createQueryFiltered($filters);
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return RepositoryInstance::getStructure()->findOperadorById($conditions, $limit, $offset);
    }

    public function structureScript($script, $connType, $connDbId)
    {
        $result = '';
        try{
            $scriptdatabase = RepositoryInstance::getStructure()->getDataBaseById($connDbId);
            if($scriptdatabase){
                $scriptdatabaseArray = json_decode($scriptdatabase[0]['CONFIG'], true) ;
                $scriptResult = Utilities::getReferenceConnectionFromDB($scriptdatabaseArray[$connType])->executeQuery($script);
            }else{
                $scriptResult = Utilities::getReferenceConnection($connType)->executeQuery($script);
            }
            if($scriptResult->rowCount() > 0) {
                $result = $scriptResult->rowCount() . 'registro(s) atualizado(s)';
            }
            $resultId = "S";
        } catch(\Exception $e) {
            $resultId = "E";
            $ignoredErrors = explode(',', Utilities::getStructureUpdateIgnoredErrorsParameter());
            $result = trim($e->getMessage());
            $isIgnoredError = false;

            foreach($ignoredErrors as $ignoredError) {
                if(strpos($ignoredError, ":")){
                    $expressions = explode(":", $ignoredError);
                    $errorCode = array_shift($expressions);
                    if(strpos($result, $errorCode)){
                        $isIgnoredError = $this->checkIgnoredErrors($result, $ignoredError, true, $expressions);
                    }
                } else{
                    $isIgnoredError = $this->checkIgnoredErrors($result, $ignoredError, false);
                }
                if($isIgnoredError){
                    break;
                }
            }

            if($isIgnoredError) {
                $result .= self::DOUBLE_BREAK_LINE . "Erro ignorado!";
                $resultId = "S";
            }
        }

        return array($result, "result" => $resultId);
    }

    public function checkIgnoredErrors($result, $ignoredError, $hasMnemonic, $expressions = null){
        if($hasMnemonic){
            foreach($expressions as $expression){
                if(strpos(strtoupper($result), strtoupper($expression))){
                    return true;
                }
            }
            return false;
        } else{
            return strpos($result, $ignoredError) ? true : false;
        }
    }

    public function getFullLog($dsDirLog){
        $fileContentString = '';
        if($dsDirLog){
            $structureRepository = RepositoryInstance::getMtDesignEstrutura();
            $oldLogFiles = $structureRepository->findOneBy(array("dsdirlog" => $dsDirLog)) ;
            if($oldLogFiles){
                $file = file_get_contents(Utilities::getFileServerFilesUrlParameter().$oldLogFiles->getDsdirlog());
            }else {
                return $fileContentString;
            }
            
            return $file;
        }else {
            return $fileContentString;
        }

    }

    public function getFullLogs($row){
        $structureRepository = RepositoryInstance::getMtDesignEstrutura();
        $logs = [];
        foreach($row['DB_ID'] as $dbId){
            if(isset($row['DSDIRLOG_' . $dbId])) {
                $dsDirLog = $row['DSDIRLOG_' . $dbId];
                $oldLogFiles = $structureRepository->findOneBy(array("dsdirlog" => $dsDirLog));
                if($oldLogFiles){
                    $file = file_get_contents(Utilities::getFileServerFilesUrlParameter().$oldLogFiles->getDsdirlog());
                    $logs['LOG_' . $dbId] = $file;
                }

            }
        }
        return $logs;

    }
    
    public function validateWebSocketDirectory($idRep){
        $currentUser = $this->environment->getCurrentUserId();
        return RepositoryInstance::getRepositorio()->validateWebSocketDirectory($currentUser, $idRep);


    }

    public function updateStructure($id, $dsDirLog, $idResultScript, $idAtivo, $nrScript, $repositorioId){

        $entityManager = GeneralProvider::getEntityManager();
        $connection    = $entityManager->getConnection();
        try{
            $connection->beginTransaction();

            $nrorg = $this->environment->getCurrentOrganizationId();
            $currentUser = $this->environment->getCurrentUserId();
            $structureRepository = RepositoryInstance::getMtDesignEstrutura();
            $structureEntity = $structureRepository->findOneBy(array('id' => $id));
            $structureScript =RepositoryInstance::getStructure()->getOrderlyScriptsById($id);
            $fullScript = "";
            foreach($structureScript as $index=>$singleScript){
                if($index == 0){
                    $fullScript = $singleScript["SCRIPT"];
                }else{
                    $fullScript .= "\n" . "/" . "\n" . $singleScript["SCRIPT"];
                }
            }

                $structureEntitySave = FactoryInstance::getMtDesignEstrutura()
                    ->updateStructure($structureEntity ,$nrorg, $currentUser, $fullScript, $dsDirLog, $idResultScript, $nrScript, $repositorioId);

                $entityManager->persist($structureEntitySave);
                $entityManager->flush();
                $connection->commit();
                return $structureEntitySave;

        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }

    }


    public function saveStructure($script, $id, $dsDesestrutura, $dsDirLog, $idResultScript, $repositorioId, $idAtivo, $nrScript, $saveType = null, $nmObjeto, $mtDb){

        $entityManager = GeneralProvider::getEntityManager();
        $connection    = $entityManager->getConnection();
        try{
            $connection->beginTransaction();

            $nrorg = $this->environment->getCurrentOrganizationId();
            $currentUser = $this->environment->getCurrentUserId();
            $structureRepository = RepositoryInstance::getMtDesignEstrutura();
            $structureEntity = $structureRepository->findOneBy(array('id' => $id));
            $script = $this->removeInvalidCharacters($script);

            if($saveType == 'EDIT' && $idResultScript == 'ES'){
                $structureEntitySave = FactoryInstance::getMtDesignEstrutura()
                    ->updateScript($structureEntity, $script, $nrorg, $currentUser, $nmObjeto, $mtDb);

                $entityManager->persist($structureEntitySave);

            }else {
                $structureEntitySave = FactoryInstance::getMtDesignEstrutura()
                    ->saveEstrutura($structureEntity ,$nrorg, $currentUser, $script, null, null,
                        $dsDirLog, $idResultScript, $repositorioId, $idAtivo, $nrScript, $dsDesestrutura, $nmObjeto, $mtDb);

                $entityManager->persist($structureEntitySave);
            }
            $entityManager->flush();
            $connection->commit();


            return $structureEntitySave;
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }

    }

    public function saveScript($scripts, $idAtivo, $structureEntitySave){
        $orderBase = null;
        $nrorg = $this->environment->getCurrentOrganizationId();
        $currentUser = $this->environment->getCurrentUserId();
        $previousId = 0;
        $i = 0;
        $orderIncrement = 0;
        $isSameFromPreviousId = false;

        $entityManager = GeneralProvider::getEntityManager();
        $connection    = $entityManager->getConnection();

        try{
            $connection->beginTransaction();
            $structureScriptRepository = RepositoryInstance::getMtDesignEstruturaScript();
            foreach($scripts as $index=>$singleScript){
                $isSameFromNext = false;
                $scriptId = empty($singleScript[0]["ID"]) ? null : $singleScript[0]["ID"];
                $scriptEntity = $structureScriptRepository->findOneBy(array('id' => $scriptId));
                if($scriptEntity){
                    $currentId = $scriptEntity->getId();

                    if($index < (count($scripts)) - 1){
                        $nextScriptEntity = $structureScriptRepository->findOneBy(array('id' => $scripts[$index + 1][0]["ID"]));
                        $isSameFromNext = ($currentId == $nextScriptEntity->getId());
                    }
                    $isSameFromPreviousId = ($currentId == $previousId) ? true : false;
                }
                $orderBase = $this->getScriptOrdenation($scriptEntity, $orderBase, $orderIncrement, $isSameFromPreviousId, $isSameFromNext, $previousId, $i);
                $scriptEntity = $isSameFromPreviousId ? null : $scriptEntity;
                $scriptCommand = $this->removeInvalidCharacters($singleScript[0]["SCRIPT"]);
                $status = $singleScript[0]["STATUS"];
                $scriptEntitySave = FactoryInstance::getMtDesignEstrutura()
                    ->saveScript($scriptEntity ,$structureEntitySave->getId(),$nrorg, $currentUser, $scriptCommand, $status,$orderBase, $idAtivo);
                $entityManager->persist($scriptEntitySave);
            }
            $entityManager->flush();
            $connection->commit();

            return $scriptEntitySave;

        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }


    }

    public function removeInvalidCharacters($scriptCommand){
        $scriptCommand = str_replace('“', '"', $scriptCommand);
        $scriptCommand = str_replace('”', '"', $scriptCommand);
        return $scriptCommand;
    }

    public function getScriptOrdenation($scriptEntity, $orderBase, &$orderIncrement, &$isSameFromPreviousId, &$isSameFromNext, &$previousId, &$i){
        if($scriptEntity){
            $currentId = $scriptEntity->getId();
            if(!$isSameFromPreviousId){
                $i = 0;
                $orderBase = $scriptEntity->getScriptOrder();
                $array = str_split("".$orderBase);
                $orderIncrement = 0;
                foreach($array as $a){
                    if($a == "0"){
                        $i++;
                    }
                }
            }
            if($isSameFromNext || $isSameFromPreviousId){
                if($i == 3){
                    $orderIncrement = 100;
                    $previousId = $currentId;
                    return $orderBase = $orderBase + $orderIncrement;
                }else if ($i == 2){
                    $orderIncrement = 10;
                    $previousId = $currentId;
                    return $orderBase = $orderBase + $orderIncrement;
                }else if ($i == 1){
                    $orderIncrement = 1;
                    $previousId = $currentId;
                    return $orderBase = $orderBase + $orderIncrement;
                }else {
                    $previousId = $currentId;
                    return $orderBase;
                }
            } else{
                return $orderBase;
            }
        }else {
            $orderIncrement = 1000;
            if($orderBase){
                return $orderBase = $orderBase + $orderIncrement;
            }else {
                return 1000;
            }
        }

    }

    public function  structuresSave($script, $id, $dsDesestrutura, $dsDirLog, $idResultScript, $repositorioId, $idAtivo, $nrScript, $saveType = null, $invalidScript, $scripts,$issueId, $nmObjeto, $mtDb)
    {
        $status = "";
        if(empty($saveType) || $saveType != 'EDIT') {
            foreach($scripts as $singleScript){
                if(($idResultScript == "PE" || $idResultScript == "" || $idResultScript == "PD" )){
                    if($singleScript[0]["STATUS"] == "E" || $invalidScript){
                    $status = "PE";
                }
            }else {
                if($singleScript[0]["STATUS"] == "E"){
                    $status = "EE";
                }
            }
        }

            if($status == ""){
                if($idResultScript == "PS" || $idResultScript == "ES"){
                    $idResultScript = "ES";
                }else{
                    $idResultScript = "PS";
                }
            }else {
                $idResultScript = $status;
            }
        }
        if(!is_array($issueId)){
            $issueId = [$issueId];
        }
        $entityManager = GeneralProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $nrorg = $this->environment->getCurrentOrganizationId();
            $currentUser = $this->environment->getCurrentUserId();
            $structureTaskEntity = null;
            $structureEntity = $this->saveStructure($script, $id, $dsDesestrutura, $dsDirLog, $idResultScript, $repositorioId, $idAtivo, $nrScript, $saveType, $nmObjeto, $mtDb);
            $mtDesignEstruraId = $structureEntity->getId();

            if ($saveType != 'EDIT' && $idResultScript != 'ES') {
                $this->saveScript($scripts, $idAtivo, $structureEntity);

                foreach ($issueId as $oneIssueId) {
                    $oneIssueId = intval($oneIssueId);
                    $structureTaskEntitySave = FactoryInstance::getMtDesignEstruturaTask()
                        ->saveStructureTask($structureTaskEntity, $currentUser, $mtDesignEstruraId, $oneIssueId, $nrorg, $nmObjeto, $mtDb);
                    $entityManager->persist($structureTaskEntitySave);
                    $entityManager->flush();
                }
            }
            if ($saveType == 'EDIT') {
                $tasks = RepositoryInstance::getMtDesignEstruturaTask()->findBy(array('mtDesignEstruraId' => $id));
                $taskArray = [];
                foreach ($tasks as $task) {
                    $currentTask = $task->getTask();
                    array_push($taskArray, $currentTask);
                    
                    if (!in_array($currentTask, $issueId)) {
                        $entityManager->remove($task);
                        $entityManager->flush();
                    }

                };

                foreach ($issueId as $oneIssueId) {
                    $oneIssueId = intval($oneIssueId);
                    if (!in_array($oneIssueId, $taskArray)) {
                        $structureTaskEntitySave = FactoryInstance::getMtDesignEstruturaTask()
                            ->saveStructureTask($structureTaskEntity, $currentUser, $mtDesignEstruraId, $oneIssueId, $nrorg, $nmObjeto, $mtDb);
                        $entityManager->persist($structureTaskEntitySave);
                        $entityManager->flush();
                    }
                }
            }
            $connection->commit();
            $conditions = " AND ID = " . $structureEntity->getId();
            $issueFilter = '';
            $issueFilterConditions = '';
            return RepositoryInstance::getStructure()->getStructure($conditions,$issueFilter,$issueFilterConditions);
            
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function  updateScript($scripts, $dsDirLog, $id, $idResultScript, $invalidScript, $idAtivo, $nrScript, $repositorioId)
    {
        $status = "";

            foreach($scripts as $singleScript){
                if(($idResultScript == "PE" || $idResultScript == "" || $idResultScript == "PD" )){
                    if($singleScript[0]["STATUS"] == "E" || $invalidScript){
                    $status = "PE";
                }
            }else {
                if($singleScript[0]["STATUS"] == "E"){
                    $status = "EE";
                }
            }
        }

        if($status == ""){
            if($idResultScript == "PS"){
                $idResultScript = "ES";
            }else{
                $idResultScript = "PS";
            }
        }else {
            $idResultScript = $status;
        }

        try {
            $structureRepository = RepositoryInstance::getMtDesignEstrutura();
            $structureEntity = $structureRepository->findOneBy(array('id' => $id));
            $this->saveScript($scripts, $idAtivo, $structureEntity);
            $this->updateStructure($id, $dsDirLog, $idResultScript, $idAtivo, $nrScript, $repositorioId);
            $issueFilter = '';
            $issueFilterConditions = '';
            $conditions =  " AND ID = " . $structureEntity->getId();
            return RepositoryInstance::getStructure()->getStructure($conditions,$issueFilter,$issueFilterConditions);
        } catch (\Exception $ex) {
            throw $ex;
        }
    }

    public function  structureUploadSave($scripts, $dsDesestrutura, $idResultScript, $repositorioId, $idAtivo,$issueId)
    {
        $entityManager = GeneralProvider::getEntityManager();
        $connection    = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $nrorg = $this->environment->getCurrentOrganizationId();
            $currentUser = $this->environment->getCurrentUserId();
            foreach($scripts as $script){
                $script = $this->removeInvalidCharacters($script);


                if($script){

                    $structureEntitySave = FactoryInstance::getMtDesignEstrutura()
                        ->uploadEstrutura($nrorg, $currentUser, $script, $idResultScript, $repositorioId, $idAtivo ,$dsDesestrutura);
                    $entityManager->persist($structureEntitySave);

                    $mtDesignEstruraId = $structureEntitySave->getId();
                    foreach($issueId as $oneIssueId){
                            $oneIssueId = intval($oneIssueId);
                            $structureTaskEntitySave = FactoryInstance::getMtDesignEstruturaTask()
                            ->saveStructureTask(null, $currentUser, $mtDesignEstruraId, $oneIssueId, $nrorg);
                            $entityManager->persist($structureTaskEntitySave);
                            $entityManager->flush();

                        }
                }
            }
            $entityManager->flush();
            $connection->commit();
            return true;
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function downloadScriptLog($fileDir, $fileName) {
        Utilities::downloadAndRemoveFile(Utilities::getFileServerFilesUrlParameter() . $fileDir, $fileName, 'text/plain', false);
    }

    public function downloadScript($id) {
        /** @var Entities\MtDesignEstrutura $structureEntity */
        $structureEntity = RepositoryInstance::getMtDesignEstrutura()->findOneBy(array('id' => $id));
        $fileDir  = Utilities::getTempLogPathParameter();
        $fileName = $structureEntity->getNrScript() . '.sql';
        $script = mb_convert_encoding( $structureEntity->getScript(), "ISO-8859-1");
        file_put_contents($fileDir . DIRECTORY_SEPARATOR . $fileName, $script);
        Utilities::downloadAndRemoveFile($fileDir . DIRECTORY_SEPARATOR . $fileName, $fileName, 'text/plain', true);
    }

    public function downloadScripList($log) {
        /** @var Entities\MtDesignEstrutura $structureEntity */
        $fileDir  = Utilities::getTempLogPathParameter();
        $fileName = 'ScriptList';
        file_put_contents($fileDir . DIRECTORY_SEPARATOR . $fileName, $log);
        Utilities::downloadAndRemoveFile($fileDir . DIRECTORY_SEPARATOR . $fileName, $fileName, 'text/plain', true);
    }

    public function scriptListInactivate($id) {
        $result   = RepositoryInstance::getStructure()->scriptListInactivate($id);
        return true;
    }

    public function getUserAndRepository($id, $dbId) {
        $filter = "AND ID = " . $id;
        $nmrepositorio   = RepositoryInstance::getStructure()->findRepositorioByIdNoPagination($filter)[0]["NMREPOSITORIO"];
        $filter = "AND ID = " . $dbId;
        $nmDb = RepositoryInstance::getStructure()->findMtDataBases($filter,null,null)[0]["NAME"];
        $currentUserName = $this->environment->getCurrentUserName();
        return array($nmrepositorio, $currentUserName, $nmDb);
    }

    public function getScriptsWithError($id) {
        $scriptsIdsWithError = [];
        $scriptsIds   = RepositoryInstance::getStructure()->findAllrelatedScripts($id);
       
        foreach($scriptsIds as $scriptsId){
            $results   = RepositoryInstance::getStructure()->findScriptsWithError($scriptsId['ID']);
            
            if($results){
                foreach($results as $result){
                    array_push($scriptsIdsWithError, $result);
                }
            }
            
        }
        return $scriptsIdsWithError;
    }

    public function getScriptNameIsInUse($name, $id, $connDb) {
        return RepositoryInstance::getStructure()->getScriptNameIsInUse($name, $id, $connDb);

    }
    public function validateScriptNameInOtherDbs($name, $connDb) {
        $dbIds = null;
        if(is_array($connDb)){
            foreach($connDb as $dbId){
                if($dbIds){
                    $dbIds .= ', ' . $dbId;
                }else{
                    $dbIds = $dbId;
                }
            }
        }
        return RepositoryInstance::getStructure()->validateScriptNameInOtherDbs($name, $dbIds);
    }

    public function getScriptList($page, $pageSize) {
        return RepositoryInstance::getStructure()->getScriptList($page, $pageSize);
    }

    public function inactivateScriptByObjName($name, $connDb) {
        $entityManager = GeneralProvider::getEntityManager();
        $structureRepository = RepositoryInstance::getMtDesignEstrutura();
        $structureEntities     = $structureRepository->findBy(array('nmObjeto' =>$name, 'mtBancoDadosId' => $connDb));
        foreach($structureEntities as $structureEntity){
            $structureEntitySave = FactoryInstance::getMtDesignEstrutura()
            ->inactivateScript($structureEntity);
            $entityManager->persist($structureEntitySave);
            $entityManager->flush();
        }
    }

    public function inactivateScriptById($id) {
        $entityManager = GeneralProvider::getEntityManager();
        $structureRepository = RepositoryInstance::getMtDesignEstrutura();
        $structureEntity     = $structureRepository->findOneBy(array('id' => $id));
        $structureEntitySave = FactoryInstance::getMtDesignEstrutura()
        ->inactivateScript($structureEntity);
        $entityManager->persist($structureEntitySave);
        $entityManager->flush();
    }

    public function inactivateScriptStructure($id) {
        $entityManager = GeneralProvider::getEntityManager();
        $connection    = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $estruturas = RepositoryInstance::getStructure()->findRelatedScript($id);
            foreach($estruturas as $estrutura){
                $this->inactivateScriptById($estrutura['ID']);
            }
            
            $connection->commit();
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }

    }

    public function inactivateScript($name, $connDbs) {
        $entityManager = GeneralProvider::getEntityManager();
        $connection    = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $structureRepository = RepositoryInstance::getMtDesignEstrutura();
            if(is_array($connDbs)){
                foreach($connDbs as $connDb){
                    $this->inactivateScriptByObjName($name, $connDb);
                }
            }else{
                $this->inactivateScriptByObjName($name, $connDbs);
            }
            
            $connection->commit();
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }

    }

    public function relateScripts($structureIds) {
        $entityManager = GeneralProvider::getEntityManager();
        $connection    = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $structureRepository = RepositoryInstance::getMtDesignEstrutura();
            foreach($structureIds as $structureId){
                $structureEntity     = $structureRepository->findOneBy(array('id' => $structureId));
                $structureEntitySave = FactoryInstance::getMtDesignEstrutura()
                ->relateScripts($structureEntity, $structureIds[0]);
                $entityManager->persist($structureEntitySave);
                $entityManager->flush();
            }
            
            $connection->commit();
        } catch (\Exception $ex) {
            $connection->rollback();
            throw $ex;
        }

    }

    public function removeStructures($scriptId){
        $structureRepository = RepositoryInstance::getMtDesignEstrutura();
        $structureScriptRepository = RepositoryInstance::getMtDesignEstruturaScript();
        $structureEntity     = $structureRepository->findOneBy(array('id' => $scriptId));
        $scriptEntitys = $structureScriptRepository->findBy(array('mtDesignEstrutId' =>$scriptId));
        $structureTaskRepository = RepositoryInstance::getMtDesignEstruturaTask();
        $structureTasks  = $structureTaskRepository->findBy(array('mtDesignEstruraId' =>$scriptId));
        foreach($scriptEntitys as $scriptEntity){
            $this->entityManager->remove($scriptEntity);
            $this->entityManager->flush();
        }
        if($structureTasks){
            foreach($structureTasks as $structureTask){
                $this->entityManager->remove($structureTask);
                $this->entityManager->flush();
            }
        }
        $this->entityManager->remove($structureEntity);
        $this->entityManager->flush();
    }

    public function removeStructure($structureId, $dbIds, $row)
    {
        $structureRepository = RepositoryInstance::getMtDesignEstrutura();
        $structureScriptRepository = RepositoryInstance::getMtDesignEstruturaScript();
        if(isset($dbIds[0])){
            foreach($dbIds as $dbId){
                if($dbId){
                    $scriptId = $row['ID_' . $dbId];
                    $this->removeStructures($scriptId);
                }
                
            }
        }else{
            $this->removeStructures($structureId);
            
        }
        
    }

    public function deleteScriptsInFileServe($imagePath)
    {
        $this->zhFileUploader->deleteFile($imagePath);
    }

}
